@@ -19,6 +19,7 @@ VIS_RESOLUTION = 0.1
 FLOAT_EPS = np.finfo(float).eps
 
 DATA_FOLDER = Path(Path(__file__).parent, "../data")
+VIS_FOLDER = Path(Path(__file__).parent, "../vis")
 
 # Diversity planner
 TSP_ELAPSED_TIME = "tsp_elapsed_time"
@@ -28,17 +29,22 @@ OPTIMIZATION_ELAPSED_TIME = "optimization_elapsed_time"
 OPTIMIZATION_ITERS = 10000
 
 ## Visualization
-VIS_LEVEL = 0
 PAUSE_DURATION = 0.1
 VIS = False
-# visualization levels, higher is more likely to get visualized
-VIS_LEVEL = 2
-VIS_LEVEL_0 = 0
-VIS_LEVEL_1 = 1
-VIS_LEVEL_2 = 2
-VIS_LEVEL_3 = 3
+FIG_SIZE = (5, 3.5)
+
+# visualization levels, lower is more likely to get visualized
+# Visualized if vis_level > (vis_level_indicator)
+VIS_LEVEL = 3
+VIS_LEVEL_0 = VIS_LEVEL > 0
+VIS_LEVEL_1 = VIS_LEVEL > 1
+VIS_LEVEL_2 = VIS_LEVEL > 2
+VIS_LEVEL_3 = VIS_LEVEL > 3
 
 # Experiments
 N_FLIGHTS = 10
 VISIT_N_LOCATIONS = 20
 N_TRIALS = 10
+
+# Neural networks
+NN_TRAINING_EPOCHS = 1000
