@@ -1,32 +1,42 @@
-import numpy as np
-from python_tsp.heuristics import solve_tsp_simulated_annealing
-from sklearn.cluster import KMeans
-from python_tsp.distances.euclidean_distance import euclidean_distance_matrix
+import time
+
 import matplotlib.pyplot as plt
+import numpy as np
+from python_tsp.distances.euclidean_distance import euclidean_distance_matrix
+from python_tsp.heuristics import solve_tsp_simulated_annealing
+from skimage.filters import gaussian
+from sklearn.cluster import KMeans
 from sklearn.preprocessing import StandardScaler
+
 from ipp_toolkit.data.MaskedLabeledImage import MaskedLabeledImage
 from ipp_toolkit.utils.optimization.optimization import topsis, quantile_solution
+from ipp_toolkit.visualization.image_data import vis_uncertainty_image
 from skimage.filters import gaussian
 from ipp_toolkit.planners.utils import (
-    compute_mask,
-    compute_interestingness_objective,
     compute_average_min_dist,
+    compute_interestingness_objective,
+    compute_mask,
     compute_n_sampled,
-    visualize_plan,
 )
+from ipp_toolkit.visualization.plan import visualize_plan
+from ipp_toolkit.visualization.optimization import visualize_pareto_front
+from ipp_toolkit.visualization.utils import show_or_save_plt
 import time
 import logging
+from ipp_toolkit.planners.candidate_location_selector import (
+    ClusteringCandidateLocationSelector,
+    GridCandidateLocationSelector,
+)
+from ipp_toolkit.config import VIS_LEVEL_1, VIS_LEVEL_2
 
-plt.rcParams["figure.figsize"] = (20, 13)
 
-from platypus import NSGAII, Problem, Binary, Real, nondominated
+from platypus import NSGAII, Binary, Problem, Real, nondominated
 
 from ipp_toolkit.config import (
     CLUSTERING_ELAPSED_TIME,
-    TSP_ELAPSED_TIME,
     OPTIMIZATION_ELAPSED_TIME,
     OPTIMIZATION_ITERS,
-    PAUSE_DURATION,
+    TSP_ELAPSED_TIME,
     VIS,
 )
 
@@ -45,7 +55,7 @@ class DiversityPlanner:
         current_location=None,
         n_spectral_bands=5,
         use_locs_for_clustering=True,
-        vis=VIS,
+        vis=VIS_LEVEL_1,
         visit_n_locations=5,
         savepath=None,
         blur_scale=5,
@@ -146,7 +156,7 @@ class DiversityPlanner:
                 labels,
                 savepath,
             )
-            self._visualize_pareto_front(
+            visualize_pareto_front(
                 pareto_results,
                 selected_objectives,
                 remove_n_sampled_locations_obj=constrain_n_samples_in_optim,
@@ -178,6 +188,8 @@ class DiversityPlanner:
         gaussian_sigma: int = 5,
         use_dense_spatial_region: bool = True,
         scaler: StandardScaler = None,
+        use_grid=False,
+        vis=VIS_LEVEL_2,
     ):
         """
         Clusters the image and then finds a large spatial region of similar appearances
@@ -196,64 +208,23 @@ class DiversityPlanner:
             centers: i,j locations of the centers
             cluster_inds: the predicted labels for each point
         """
-        start_time = time.time()
-        kmeans = KMeans(n_clusters=n_clusters)
-        if scaler is None:
-            scaler = StandardScaler()
-            # Normalize features elementwise
-            features = scaler.fit_transform(features)
+        if use_grid:
+            selector = GridCandidateLocationSelector(img_size=img_size)
         else:
-            features = scaler.transform(features)
-
-        if max_fit_points is None:
-            # Fit on all the points
-            kmeans.fit(features)
-        else:
-            # Fit on a subset of points
-            sample_inds = np.random.choice(
-                features.shape[0], size=(max_fit_points), replace=False
+            selector = ClusteringCandidateLocationSelector(
+                img_size=img_size,
+                max_fit_points=max_fit_points,
+                gaussian_sigma=gaussian_sigma,
+                use_dense_spatial_region=use_dense_spatial_region,
+                scaler=scaler,
             )
-            feature_subset = features[sample_inds, :]
-            kmeans.fit(feature_subset)
-        # Predict the cluster membership for each data point
-        cluster_inds = kmeans.predict(features)
+        centers, cluster_inds, scaler, elapsed_time = selector.select_locations(
+            features=features, mask=mask, n_clusters=n_clusters, loc_samples=loc_samples
+        )
+        self.log_dict[CLUSTERING_ELAPSED_TIME] = elapsed_time
 
-        # Two approaches, one is a dense spatial cluster
-        # the other is the point nearest the k-means centroid
-        if use_dense_spatial_region:
-            # Build an array where each channel is a binary map for one class
-            per_class_layers = np.zeros(
-                (img_size[0], img_size[1], n_clusters), dtype=bool
-            )
-            # Populate the binary map
-            per_class_layers[
-                loc_samples[:, 0].astype(int),
-                loc_samples[:, 1].astype(int),
-                cluster_inds,
-            ] = True
-            # Smooth the binary map layer-wise
-            smoothed_layers = [
-                gaussian(per_class_layers[..., i], gaussian_sigma)
-                for i in range(n_clusters)
-            ]
-            masked_smoothed_layers = []
-            for l in smoothed_layers:
-                l[np.logical_not(mask)] = 0
-                masked_smoothed_layers.append(l)
-            # Find the argmax location for each smoothed binary map
-            # This is just fancy indexing for that
-            centers = [
-                np.unravel_index(msi.flatten().argmax(), msi.shape)
-                for msi in masked_smoothed_layers
-            ]
-            centers = np.vstack(centers)
-        else:
-            # Obtain the distance to each class centroid
-            center_dists = kmeans.transform(features)
-            closest_points = np.argmin(center_dists, axis=0)
-            centers = loc_samples[closest_points].astype(int)
-
-        self.log_dict[CLUSTERING_ELAPSED_TIME] = time.time() - start_time
+        if vis:
+            selector.vis(self.world_data)
         return centers, cluster_inds, scaler
 
     def _get_candidate_location_features(
@@ -423,66 +394,6 @@ class DiversityPlanner:
 
         return final_mask, final_objectives
 
-    def _visualize_pareto_front(
-        self,
-        pareto_solutions,
-        selected_objectives,
-        pause_duration=PAUSE_DURATION,
-        remove_n_sampled_locations_obj: bool = False,
-        labels=(
-            "Number of sampled locations",
-            "Average distance of unsampled locations",
-            "Sum interestingness score",
-        ),
-    ):
-        """
-        Args:
-            show_n_sampled_locations: should you show the sampled locations objective
-
-        """
-        # close existing figures
-        plt.close()
-        plt.clf()
-
-        pareto_objectives = np.array([s.objectives for s in pareto_solutions])
-
-        # If the number of sample locations is constrained, this objective is not present
-        if remove_n_sampled_locations_obj:
-            # pareto_objectives = pareto_objectives[:, 1:]
-            labels = labels[1:]
-
-        dimensionality = pareto_objectives.shape[1]
-        if dimensionality == 1:
-            logging.info(
-                f"Objective value for one-dim problem {pareto_objectives[0, 0]}"
-            )
-            return
-        if dimensionality == 2:
-            # Normal 2d plot
-            plt.xlabel(labels[0])
-            plt.ylabel(labels[1])
-            ax = plt
-
-        elif dimensionality == 3:
-            # Set up 3d plot
-            ax = plt.figure().add_subplot(projection="3d")
-            # Plot z axis which would otherwise be missing
-            ax.set_xlabel(labels[0])
-            ax.set_ylabel(labels[1])
-            ax.set_zlabel(labels[2])
-
-        else:
-            raise ValueError(
-                f"Cannot show problem with {dimensionality} dimensions, only 3 or fewer"
-            )
-        # Show all candidate objectives
-        ax.scatter(*pareto_objectives.T, label="Candidate solutions")
-        # Show the selected location
-        ax.scatter(*selected_objectives, c="r", s=50, label="Chosen solution")
-
-        plt.legend()
-        plt.pause(pause_duration)
-
 
 class BatchDiversityPlanner(DiversityPlanner):
     def __init__(
@@ -545,11 +456,12 @@ class BatchDiversityPlanner(DiversityPlanner):
         self,
         interestingness_image: np.ndarray = None,
         current_location=None,
-        vis=VIS,
+        vis=VIS_LEVEL_1,
         visit_n_locations=5,
         savepath=None,
         constrain_n_samples_in_optim: bool = True,
         n_optimization_iters=OPTIMIZATION_ITERS,
+        vis_uncertainty=VIS_LEVEL_2,
         **kwargs,
     ):
         """
@@ -571,6 +483,9 @@ class BatchDiversityPlanner(DiversityPlanner):
         if interestingness_image is not None:
             self.interestingness_image = interestingness_image
 
+        if vis_uncertainty and self.interestingness_image is not None:
+            vis_uncertainty_image(interestingness_image, "vis/uncertainty.png")
+
         # Get the candidate locations
         if self.cluster_labels is None or self.candidate_locations is None:
             print("computing new cluster centers")
